/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;
import java.awt.Frame;
import java.util.Properties;
import java.io.*;

/**
 * Provides configuration details to Conient
 * This class is a Singleton class.  It handles all the configuration
 * for Conient.  Once a connection has been made, it is told that 
 * it can get configuration from the server, thus allowing other components
 * access to the servers configuration.  It also shows the ConfigurationDialog
 * on request, which allows local configuration options to be changed.
 *
 * It also has support for a variety of methods of saving the config in 
 * different files and in the default file.
 *
 * @author  $Author: tdb $
 * @version $Id: Configuration.java,v 1.13 2003/02/25 22:14:20 tdb Exp $
 */
public class Configuration {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.13 $";
    
    public static final String CONFIG_HEADER = 
        "!!! Conient Local Configuration File !!!\n" +
        "#-----------------------------------------\n" +
        "# This file was auto-generated by Conient.\n" +
        "# It is recommended that you use the GUI\n" +
        "# configuration facility to make changes\n" +
        "# to this file.\n#";
    
//---STATIC METHODS---

    /**
     * Creates and initialises a the Configuration
     * system for Conient.  This calls the private
     * constructor and ensures this class is a singleton.
     *
     * @param configFile the path to the file that this configuration should load
     */
    public static void initialise(String configFile) {
        _instance = new Configuration(configFile);
    }

    /**
     * Returns the singleton instance of this
     * class.
     * This will throw a runtime exception if it 
     * is called at the wrong time!
     *
     * @return the singleton instance
     */
    public static Configuration getInstance() {
        if (_instance == null) {
            throw new RuntimeException("Configuration class requested but HASN'T been initialised!");
        }
        return _instance;
    }

//---CONSTRUCTORS---

    /**
     * The private constructor, ensures that this is a singleton.
     * Simply reads in the local configuration from the file.
     * Then sets the current properties to be this configuration.
     * Later the system will obtain further configuration from the
     * server.
     *
     * @param configFile the path to the file that this configuration should load
     */
    private Configuration(String configFile) {
        _configFile = new File(configFile);
        _defaultConfigFile = _configFile;
        try {
            _properties = readFileConfiguration(_configFile);
        } catch (FileNotFoundException e) {
            JOptionPane.showMessageDialog(null, "Configuration file not found - " + _configFile.getName(), "Configuration Error", JOptionPane.ERROR_MESSAGE);
            System.exit(1);
        } catch (IOException e) {
            JOptionPane.showMessageDialog(null, "Unable to read configuration file - " + _configFile.getName() + "\nReason: " + e, "Configuration Error", JOptionPane.ERROR_MESSAGE);
            System.exit(1);
        }
    }

//---PUBLIC METHODS---

    /**
     * This method is called by any part of the system
     * that requires configuration from the server.
     *
     * It should be passed the configuration name:
     * eg, "Host.raptor.ukc.ac.uk"
     * and the property required:
     * eg, "Host.UDPUpdateTime"
     *
     * This method will then ask the ConnectionHandler to
     * talk to the server and return the property.  If the server
     * fails to get the property, or the ConnectionHandler
     * is not started, this method returns null.
     */
    public String getServerProperty(String configName, String propertyName) {
        String property = null;
        if (_connectionHandler != null) {
            property = _connectionHandler.getConfigFromServer(configName, propertyName);
        }
        return property;
    }
    
    /**
     * Tells the configuration class that it should
     * perform user re-configuration through displaying
     * a gui.  This passes control to a dialog to 
     * handle re-configuration.  Basically this is
     * the nicer alternative to hacking the config
     * file.
     * The ConientConfiguration class is responsible
     * for handling the re-configuration.
     */
    public void GUIReconfiguration() {
        ConfigurationDialog conf = new ConfigurationDialog();
    }
    
    /**
     * Prompts the user for a box to save the configuration
     * to a specific filename.
     */
    public void saveNewConfiguration() {
        fc.setCurrentDirectory(_configFile.getParentFile());
        
        int returnVal = fc.showSaveDialog(Conient.getFrame());
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            _configFile = fc.getSelectedFile();
            _usingSpecificConfig = true;
            saveConfiguration();
        }
    }
    
    /**
     * This method saves the currently saves the current
     * config, if the config has not been saved before
     * at it is not the default, then it prompts for a 
     * file name by calling saveNewConfiguration().
     */
    public void saveConfiguration() {
        saveFileConfiguration(_configFile);
    }
    
    /**
     * This method saves the currently loaded config as
     * the default config
     */
    public void saveDefaultConfiguration() {
        saveFileConfiguration(_defaultConfigFile);
    }
    
    /**
     * Loads in a configuration that is chosen by the 
     * user after displaying a dialog.
     */    
    public void loadConfiguration() {
        fc.setCurrentDirectory(_configFile.getParentFile());
        int returnVal = fc.showOpenDialog(Conient.getFrame());
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            _configFile = fc.getSelectedFile();
            _usingSpecificConfig = true;
            try {
                _properties = readFileConfiguration(_configFile);
                JOptionPane.showMessageDialog(null, "Configuration sucessfully read in - " + _configFile.getName(), "Configuration Loaded", JOptionPane.INFORMATION_MESSAGE);
            } catch (FileNotFoundException e) {
                JOptionPane.showMessageDialog(null, "Configuration file not found - " + _configFile.getName(), "Configuration Error", JOptionPane.ERROR_MESSAGE);
            } catch (IOException e) {
                JOptionPane.showMessageDialog(null, "Unable to read configuration file - " + _configFile.getName() + "\nReason: " + e, "Configuration Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
//---PRIVATE METHODS---

    /**
     * Reads in the specified file and parses
     * its properties.
     *
     * @param configFile the path of the file to be read
     * @return the parsed properties
     */
    private Properties readFileConfiguration(File inputFile) throws FileNotFoundException, IOException {
        Properties configHolder = new Properties();
        configHolder.load(new FileInputStream(inputFile));
        return configHolder;
    }
    
    /**
     * This method writes out the current configuration
     * to a file using the Properties.store() method.
     * 
     * It uses the CONFIG_HEADER attribute to head the file.
     *
     * @param outputfile the file to write the configurtion to
     */
    private void saveFileConfiguration(File outputFile) {
        try {
            _properties.store(new FileOutputStream(outputFile), CONFIG_HEADER);
            JOptionPane.showMessageDialog(null, "Configuration written out - " + _configFile.getName(), "Configuration Saved", JOptionPane.INFORMATION_MESSAGE);
        } catch (IOException e) {
            JOptionPane.showMessageDialog(null, "Unable to write default configuration file - " + outputFile.getName() + "\nReason: " + e, "Configuration Error", JOptionPane.ERROR_MESSAGE);
        }
    }

//---ACCESSOR/MUTATOR METHODS---

    /**
     * A wrapper for java.util.Properties.getProperty
     * When given a key it returns the value of that key
     * ie, key = value
     *
     * @param key the key the value of which is wanted
     */
    public String getProperty(String key) {
        String property = _properties.getProperty(key); 
        if (property == null) {
            return "";
        }
        return property;
    }
    
    /**
     * A wrapper for java.util.Properties.setProperty
     * When given a key and a value, it writes it to the
     * current properties.
     *
     * @param key the key to write
     * @param value the value to assign it
     */
    public void setProperty(String key, String value) {
        _properties.setProperty(key, value);
    }
    
    /**
     * Returns whether a specific config is in use
     * rather than a default one
     *
     * @return yay or nay
     */
    public boolean getUsingSpecificConfig() {
        return _usingSpecificConfig;
    }

    /**
     * When the connection handler class starts up it notifys this class
     * that it is up, so that we can obtain configuration through the
     * open connections, should there be any.
     *
     * @param connectionHandler a handle on the instance of the connnection handler
     */
    public void setConnectionHandler(ConnectionHandler connectionHandler) {
        _connectionHandler = connectionHandler;
    }

//---ATTRIBUTES---

    /**
     * The current configuration in use
     */
    private Properties _properties = null;
    
    /**
     * A reference to the ConnectionHandler, this allows
     * this class to obtain configuration from the server
     */
    private ConnectionHandler _connectionHandler = null;
    
    /**
     * A value to indicate whether a specific config is in use 
     * or not
     */
    private boolean _usingSpecificConfig = false;
    
    /**
     * The file that the current configuration is loaded from
     */
    private File _configFile;
    
    /**
     * The file containing the default configuration
     */
    private File _defaultConfigFile;
    
    /**
     * A file filter for the file chooser dialog boxes
     */
    private final FileFilter filter = new SuffixFileFilter("conf", "Conient Configuration Files");
    
    /**
     * A file chooser to prompt for file names when saving configuration
     */
    private final JFileChooser fc = new JFileChooser();
    {
        fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fc.setMultiSelectionEnabled(false);
        fc.setFileFilter(new FileFilter() {
            public boolean accept(File f) {
                return (f != null) && (f.isDirectory() || filter.accept(f));
            }
            public String getDescription() {
                return "Folders and Conient Configuration Files";
            }
        });
        fc.addChoosableFileFilter(filter);
    }
    
//---STATIC ATTRIBUTES---

    /**
     * The reference to the singleton instance of this class
     */
    private static Configuration _instance = null;
    
//---INNER CLASSES---

    /**
     * An inner class for the file filter.
     * This allows filters to be defined for files based on their
     * suffix.
     */
    private class SuffixFileFilter extends FileFilter {
        public SuffixFileFilter(String suffix, String description) {
            if (suffix.charAt(0) != '.') {
                _suffix = '.' + suffix;
            } else {
                _suffix = suffix;
            }
            _description = description;
        }
        
        public boolean accept(File f) {
            boolean ok = false;
            if (f != null) {
                final String name = f.getName();
                if (name != null) {
                    final int nameLength = name.length();
                    int dotIndex = name.lastIndexOf(".");
                    ok = (dotIndex >= 1) &&
                        getSuffix().equalsIgnoreCase(name.substring(dotIndex));
                }
            }
            return ok;
        }
        
        public String getDescription() {
            return _description;
        }
        
        public String getSuffix() {
            return _suffix;
        }
        
        private final String _suffix, _description;
    }
            
}
